@@ -1,10 +1,22 @@
-#include "FileZilla.h"
-#include "logging_private.h"
+#include <filezilla.h>
 #include "ControlSocket.h"
-#include <idna.h>
-#include "asynchostresolver.h"
 #include "directorycache.h"
+#include "local_filesys.h"
+#include "local_path.h"
+#include "logging_private.h"
+#include "proxy.h"
 #include "servercapabilities.h"
+#include "sizeformatting_base.h"
+
+#include <wx/file.h>
+#include <wx/filename.h>
+
+#include <idna.h>
+extern "C" {
+#include <idn-free.h>
+}
+
+#include <errno.h>
 
 DECLARE_EVENT_TYPE(fzOBTAINLOCK, -1)
 DEFINE_EVENT_TYPE(fzOBTAINLOCK)
@@ -28,6 +40,8 @@ COpData::COpData(enum Command op_Id)
 	pNextOpData = 0;
 
 	waitForAsyncRequest = false;
+
+	holdsLock = false;
 }
 
 COpData::~COpData()
@@ -40,6 +54,7 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate *pEngine)
 {
 	m_pEngine = pEngine;
 	m_pCurOpData = 0;
+	m_nOpState = 0;
 	m_pCurrentServer = 0;
 	m_pTransferStatus = 0;
 	m_transferStatusSendState = 0;
@@ -48,7 +63,7 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate *pEngine)
 	m_useUTF8 = false;
 
 	m_timer.SetOwner(this);
-	
+
 	m_closed = false;
 
 	m_invalidateCurrentPath = false;
@@ -78,6 +93,53 @@ enum Command CControlSocket::GetCurrentCommandId() const
 	return m_pEngine->GetCurrentCommandId();
 }
 
+void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData)
+{
+	if (m_pTransferStatus && (nErrorCode == FZ_REPLY_OK || m_pTransferStatus->madeProgress))
+	{
+		int elapsed = wxTimeSpan(wxDateTime::Now() - m_pTransferStatus->started).GetSeconds().GetLo();
+		if (elapsed <= 0)
+			elapsed = 1;
+		wxString time = wxString::Format(
+			wxPLURAL("%d second", "%d seconds", elapsed),
+			elapsed);
+		
+		wxLongLong transferred = m_pTransferStatus->currentOffset - m_pTransferStatus->startOffset;
+		wxString size = CSizeFormatBase::Format(m_pEngine->GetOptions(), transferred, true);
+
+		MessageType msgType = ::Error;
+		wxString msg;
+		if (nErrorCode == FZ_REPLY_OK)
+		{
+			msgType = Status;
+			msg = _("File transfer successful, transferred %s in %s");
+		}
+		else if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+			msg = _("File transfer aborted by user after transferring %s in %s");
+		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
+			msg = _("Critical file transfer error after transferring %s in %s");
+		else
+			msg = _("File transfer failed after transferring %s in %s");
+		LogMessage(msgType, msg, size.c_str(), time.c_str());
+	}
+	else
+	{
+		if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+			LogMessage(::Error, _("File transfer aborted by user"));
+		else if (nErrorCode == FZ_REPLY_OK)
+		{
+			if (pData->transferInitiated)
+				LogMessage(Status, _("File transfer successful"));
+			else
+				LogMessage(Status, _("File transfer skipped"));
+		}
+		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
+			LogMessage(::Error, _("Critical file transfer error"));
+		else
+			LogMessage(::Error, _("File transfer failed"));
+	}
+}
+
 int CControlSocket::ResetOperation(int nErrorCode)
 {
 	LogMessage(Debug_Verbose, _T("CControlSocket::ResetOperation(%d)"), nErrorCode);
@@ -87,10 +149,8 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		LogMessage(::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
 	}
 
-	if (m_pCurOpData && m_pCurOpData->opId != cmd_rawtransfer)
-	{
+	if (m_pCurOpData && m_pCurOpData->holdsLock)
 		UnlockCache();
-	}
 
 	if (m_pCurOpData && m_pCurOpData->pNextOpData)
 	{
@@ -102,14 +162,17 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			nErrorCode == FZ_REPLY_ERROR ||
 			nErrorCode == FZ_REPLY_CRITICALERROR)
 		{
-			return SendNextCommand(nErrorCode);
+			return ParseSubcommandResult(nErrorCode);
 		}
 		else
 			return ResetOperation(nErrorCode);
 	}
 
-	if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
+	if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR &&
+		(!m_pCurOpData || m_pCurOpData->opId != cmd_transfer))
+	{
 		LogMessage(::Error, _("Critical error"));
+	}
 
 	if (m_pCurOpData)
 	{
@@ -148,10 +211,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 							m_pEngine->SendDirectoryListingNotification(pData->remotePath, false, true, false);
 					}
 				}
-				if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
-					LogMessage(::Error, _("Transfer aborted by user"));
-				else if (nErrorCode == FZ_REPLY_OK)
-					LogMessage(Status, _("File transfer successful"));
+				LogTransferResultMessage(nErrorCode, pData);
 			}
 			break;
 		default:
@@ -179,16 +239,17 @@ int CControlSocket::ResetOperation(int nErrorCode)
 
 int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
+	LogMessage(Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
 	if (m_closed)
 	{
 		wxASSERT(!m_pCurOpData);
 		return nErrorCode;
 	}
-	
+
 	m_closed = true;
 
 	nErrorCode = ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | nErrorCode);
-	
+
 	delete m_pCurrentServer;
 	m_pCurrentServer = 0;
 
@@ -196,8 +257,7 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 }
 
 wxString CControlSocket::ConvertDomainName(wxString domain)
-{return domain;
-/*
+{
 	const wxWCharBuffer buffer = wxConvCurrent->cWX2WC(domain);
 
 	int len = 0;
@@ -218,8 +278,8 @@ wxString CControlSocket::ConvertDomainName(wxString domain)
 	delete [] utf8;
 
 	wxString result = wxConvCurrent->cMB2WX(output);
-	free(output);
-	return result;*/
+	idn_free(output);
+	return result;
 }
 
 void CControlSocket::Cancel()
@@ -255,6 +315,7 @@ void CControlSocket::InitTransferStatus(wxFileOffset totalSize, wxFileOffset sta
 	m_pTransferStatus->totalSize = totalSize;
 	m_pTransferStatus->startOffset = startOffset;
 	m_pTransferStatus->currentOffset = startOffset;
+	m_pTransferStatus->madeProgress = false;
 }
 
 void CControlSocket::SetTransferStatusStartTime()
@@ -265,6 +326,14 @@ void CControlSocket::SetTransferStatusStartTime()
 	m_pTransferStatus->started = wxDateTime::Now();
 }
 
+void CControlSocket::SetTransferStatusMadeProgress()
+{
+	if (!m_pTransferStatus)
+		return;
+
+	m_pTransferStatus->madeProgress = true;
+}
+
 void CControlSocket::UpdateTransferStatus(wxFileOffset transferredBytes)
 {
 	if (!m_pTransferStatus)
@@ -315,22 +384,22 @@ bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, con
 		int pos2 = reply.Find('"', true);
 		if (pos1 == -1 || pos1 >= pos2)
 		{
-			int pos1 = reply.Find('\'');
-			int pos2 = reply.Find('\'', true);
+			pos1 = reply.Find('\'');
+			pos2 = reply.Find('\'', true);
 
 			if (pos1 != -1 && pos1 < pos2)
 				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server sending single-quoted path instead of double-quoted path."));
 		}
 		if (pos1 == -1 || pos1 >= pos2)
 		{
-			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No quoted path found in pwd reply, trying first token as path"));
+			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
 			pos1 = reply.Find(' ');
 			if (pos1 != -1)
 			{
-				pos2 = reply.Mid(pos1 + 1).Find(' ');
-				if (pos2 == -1)
-					pos2 = (int)reply.Length();
-				reply = reply.Mid(pos1 + 1, pos2 - pos1 - 1);
+				reply = reply.Mid(pos1 + 1);
+				pos2 = reply.Find(' ');
+				if (pos2 != -1)
+					reply = reply.Left(pos2);
 			}
 			else
 				reply = _T("");
@@ -380,7 +449,12 @@ int CControlSocket::CheckOverwriteFile()
 	bool dirDidExist;
 	bool matchedCase;
 	CDirectoryCache cache;
-	bool found = cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dirDidExist, matchedCase);
+	CServerPath remotePath;
+	if (pData->tryAbsolutePath || m_CurrentPath.IsEmpty())
+		remotePath = pData->remotePath;
+	else
+		remotePath = m_CurrentPath;
+	bool found = cache.LookupFile(entry, *m_pCurrentServer, remotePath, pData->remoteFile, dirDidExist, matchedCase);
 
 	// Ignore entries with wrong case
 	if (found && !matchedCase)
@@ -402,6 +476,13 @@ int CControlSocket::CheckOverwriteFile()
 	pNotification->remoteSize = pData->remoteFileSize;
 	pNotification->ascii = !pData->transferSettings.binary;
 
+	if (pData->download && pNotification->localSize != -1)
+		pNotification->canResume = true;
+	else if (!pData->download && pNotification->remoteSize != -1)
+		pNotification->canResume = true;
+	else
+		pNotification->canResume = false;
+
 	wxStructStat buf;
 	int result;
 	result = wxStat(pData->localFile, &buf);
@@ -419,7 +500,7 @@ int CControlSocket::CheckOverwriteFile()
 	{
 		if (!pData->fileTime.IsValid())
 		{
-			if (entry.hasDate)
+			if (entry.has_date())
 			{
 				pNotification->remoteTime = entry.time;
 				pData->fileTime = entry.time;
@@ -427,16 +508,15 @@ int CControlSocket::CheckOverwriteFile()
 		}
 	}
 
-	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
-	pData->waitForAsyncRequest = true;
-
-	m_pEngine->AddNotification(pNotification);
+	SendAsyncRequest(pNotification);
 
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-CFileTransferOpData::CFileTransferOpData() :
+CFileTransferOpData::CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path) :
 	COpData(cmd_transfer),
+ 	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
+	download(is_download),
 	localFileSize(-1), remoteFileSize(-1),
 	tryAbsolutePath(false), resume(false), transferInitiated(false)
 {
@@ -465,7 +545,7 @@ wxString CControlSocket::ConvToLocal(const char* buffer)
 			m_useUTF8 = false;
 		}
 	}
-	
+
 	if (m_pCSConv)
 	{
 		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv);
@@ -593,6 +673,9 @@ void CControlSocket::OnTimer(wxTimerEvent& event)
 	if (m_pCurOpData && m_pCurOpData->waitForAsyncRequest)
 		return;
 
+	if (IsWaitingForLock())
+		return;
+
 	if (m_stopWatch.Time() > (timeout * 1000))
 	{
 		LogMessage(::Error, _("Connection timed out"));
@@ -620,25 +703,32 @@ void CControlSocket::SetWait(bool wait)
 		m_timer.Stop();
 }
 
-int CControlSocket::SendNextCommand(int prevResult /*=FZ_REPLY_OK*/)
+int CControlSocket::SendNextCommand()
 {
-	ResetOperation(prevResult);
+	ResetOperation(FZ_REPLY_INTERNALERROR);
+	return FZ_REPLY_ERROR;
+}
+
+int CControlSocket::ParseSubcommandResult(int prevResult)
+{
+	ResetOperation(FZ_REPLY_INTERNALERROR);
 	return FZ_REPLY_ERROR;
 }
 
 const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockStatus()
 {
 	std::list<t_lockInfo>::iterator iter;
-	for (iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); iter++)
+	for (iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); ++iter)
 		if (iter->pControlSocket == this)
 			break;
 
 	return iter;
 }
 
-bool CControlSocket::TryLockCache(const CServerPath& directory)
+bool CControlSocket::TryLockCache(enum locking_reason reason, const CServerPath& directory)
 {
 	wxASSERT(m_pCurrentServer);
+	wxASSERT(m_pCurOpData);
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
@@ -647,49 +737,121 @@ bool CControlSocket::TryLockCache(const CServerPath& directory)
 		info.directory = directory;
 		info.pControlSocket = this;
 		info.waiting = true;
+		info.reason = reason;
+		info.lockcount = 0;
 		m_lockInfoList.push_back(info);
 		own = --m_lockInfoList.end();
 	}
 	else
 	{
+		if (own->lockcount)
+		{
+			if (!m_pCurOpData->holdsLock)
+			{
+				m_pCurOpData->holdsLock = true;
+				own->lockcount++;
+			}
+			return true;
+		}
 		wxASSERT(own->waiting);
+		wxASSERT(own->reason == reason);
 	}
 
+	// Needs to be set in any case so that ResetOperation 
+	// unlocks or cancels the lock wait
+	m_pCurOpData->holdsLock = true;
+
 	// Try to find other instance holding the lock
-	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; iter++)
+	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; ++iter)
 	{
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
+		if (directory != iter->directory)
+			continue;
+		if (reason != iter->reason)
+			continue;
 
-		if (directory == iter->directory)
-		{
-			// Some other instance is holding the lock
-			return false;
-		}
+		// Some other instance is holding the lock
+		return false;
 	}
 
+	own->lockcount++;
 	own->waiting = false;
 	return true;
 }
 
+bool CControlSocket::IsLocked(enum locking_reason reason, const CServerPath& directory)
+{
+	wxASSERT(m_pCurrentServer);
+
+	std::list<t_lockInfo>::iterator own = GetLockStatus();
+	if (own != m_lockInfoList.end())
+		return true;
+
+	// Try to find other instance holding the lock
+	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; ++iter)
+	{
+		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
+			continue;
+		if (directory != iter->directory)
+			continue;
+		if (reason != iter->reason)
+			continue;
+
+		// Some instance is holding the lock
+		return true;
+	}
+
+	return false;
+}
+
 void CControlSocket::UnlockCache()
 {
+	if (!m_pCurOpData || !m_pCurOpData->holdsLock)
+		return;
+	m_pCurOpData->holdsLock = false;
+
 	std::list<t_lockInfo>::iterator iter = GetLockStatus();
 	if (iter == m_lockInfoList.end())
 		return;
 
+	wxASSERT(!iter->waiting || iter->lockcount == 0);
+	if (!iter->waiting)
+	{
+		iter->lockcount--;
+		wxASSERT(iter->lockcount >= 0);
+		if (iter->lockcount)
+			return;
+	}
+
 	CServerPath directory = iter->directory;
+	enum locking_reason reason = iter->reason;
+
 	m_lockInfoList.erase(iter);
 
 	// Find other instance waiting for the lock
-	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); iter++)
+	if (!m_pCurrentServer)
 	{
+		LogMessage(Debug_Warning, _T("UnlockCache called with !m_pCurrentServer"));
+		return;
+	}
+	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); ++iter)
+	{
+		if (!iter->pControlSocket->m_pCurrentServer)
+		{
+			LogMessage(Debug_Warning, _T("UnlockCache found other instance with !m_pCurrentServer"));
+			continue;
+		}
+
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
 
 		if (iter->directory != directory)
 			continue;
 
+		if (iter->reason != reason)
+			continue;
+
 		// Send notification
 		wxCommandEvent evt(fzOBTAINLOCK);
 		iter->pControlSocket->AddPendingEvent(evt);
@@ -697,44 +859,42 @@ void CControlSocket::UnlockCache()
 	}
 }
 
-bool CControlSocket::ObtainLockFromEvent()
+enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 {
+	if (!m_pCurOpData)
+		return lock_unknown;
+
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
-		return false;
+		return lock_unknown;
 
 	if (!own->waiting)
-		return false;
+		return lock_unknown;
 
-	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; iter++)
+	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; ++iter)
 	{
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
 
-		if (iter->directory == own->directory)
-			return false;
+		if (iter->directory != own->directory)
+			continue;
+
+		if (iter->reason != own->reason)
+			continue;
+
+		// Another instance comes before us
+		return lock_unknown;
 	}
 
 	own->waiting = false;
+	own->lockcount++;
 
-	return true;
-}
-
-bool CControlSocket::HasLock()
-{
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end())
-		return false;
-
-	if (own->waiting)
-		return false;
-
-	return true;
+	return own->reason;
 }
 
 void CControlSocket::OnObtainLock(wxCommandEvent& event)
 {
-	if (!ObtainLockFromEvent())
+	if (ObtainLockFromEvent() == lock_unknown)
 		return;
 
 	SendNextCommand();
@@ -742,6 +902,15 @@ void CControlSocket::OnObtainLock(wxCommandEvent& event)
 	UnlockCache();
 }
 
+bool CControlSocket::IsWaitingForLock()
+{
+	std::list<t_lockInfo>::iterator own = GetLockStatus();
+	if (own == m_lockInfoList.end())
+		return false;
+
+	return own->waiting == true;
+}
+
 void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 {
 	wxASSERT(!path.IsEmpty());
@@ -765,32 +934,46 @@ wxTimeSpan CControlSocket::GetTimezoneOffset()
 	int seconds = 0;
 	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) != yes)
 		return wxTimeSpan();
-	
+
 	return wxTimeSpan(0, 0, seconds);
 }
 
+void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
+{
+	wxASSERT(pNotification);
+
+	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
+
+	if (m_pCurOpData)
+		m_pCurOpData->waitForAsyncRequest = true;
+	m_pEngine->AddNotification(pNotification);
+}
+
 // ------------------
 // CRealControlSocket
 // ------------------
 
-BEGIN_EVENT_TABLE(CRealControlSocket, CControlSocket)
-	EVT_SOCKET(wxID_ANY, CRealControlSocket::OnSocketEvent)
-END_EVENT_TABLE()
-
 CRealControlSocket::CRealControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CControlSocket(pEngine), wxSocketClient(wxSOCKET_NOWAIT)
+	: CControlSocket(pEngine)
 {
-	m_pBackend = new CSocketBackend(this, this);
+	m_pSocket = new CSocket(this);
+
+	m_pBackend = new CSocketBackend(this, m_pSocket);
+	m_pProxyBackend = 0;
 
 	m_pSendBuffer = 0;
 	m_nSendBufferLen = 0;
-	m_onConnectCalled = false;
 }
 
 CRealControlSocket::~CRealControlSocket()
 {
+	m_pSocket->Close();
+	if (m_pProxyBackend && m_pProxyBackend != m_pBackend)
+		delete m_pProxyBackend;
 	delete m_pBackend;
 	m_pBackend = 0;
+
+	delete m_pSocket;
 }
 
 bool CRealControlSocket::Send(const char *buffer, int len)
@@ -807,29 +990,30 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 	}
 	else
 	{
-		m_pBackend->Write(buffer, len);
-		int numsent = 0;
-		if (m_pBackend->Error())
+		int error;
+		int written = m_pBackend->Write(buffer, len, error);
+		if (written < 0)
 		{
-			if (m_pBackend->LastError() != wxSOCKET_WOULDBLOCK)
+			if (error != EAGAIN)
 			{
+				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
+				LogMessage(::Error, _("Disconnected from server"));
 				DoClose();
 				return false;
 			}
+			written = 0;
 		}
-		else
-			numsent = m_pBackend->LastCount();
 
-		if (numsent)
-			m_pEngine->SetActive(false);
+		if (written)
+			m_pEngine->SetActive(CFileZillaEngine::send);
 
-		if (numsent < len)
+		if (written < len)
 		{
 			char *tmp = m_pSendBuffer;
-			m_pSendBuffer = new char[m_nSendBufferLen + len - numsent];
+			m_pSendBuffer = new char[m_nSendBufferLen + len - written];
 			memcpy(m_pSendBuffer, tmp, m_nSendBufferLen);
-			memcpy(m_pSendBuffer + m_nSendBufferLen, buffer, len - numsent);
-			m_nSendBufferLen += len - numsent;
+			memcpy(m_pSendBuffer + m_nSendBufferLen, buffer, len - written);
+			m_nSendBufferLen += len - written;
 			delete [] tmp;
 		}
 	}
@@ -837,33 +1021,50 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 	return true;
 }
 
-void CRealControlSocket::OnSocketEvent(wxSocketEvent &event)
+void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
 {
 	if (!m_pBackend)
 		return;
 
-	if (event.GetId() != m_pBackend->GetId())
-		return;
-
-	switch (event.GetSocketEvent())
+	switch (event.GetType())
 	{
-	case wxSOCKET_CONNECTION:
-		m_onConnectCalled = true;
-		OnConnect();
-		break;
-	case wxSOCKET_INPUT:
-		if (!m_onConnectCalled)
+	case CSocketEvent::hostaddress:
 		{
-			m_onConnectCalled = true;
+			const wxString& address = event.GetData();
+			LogMessage(Status, _("Connecting to %s..."), address.c_str()); 
+		}
+		break;
+	case CSocketEvent::connection_next:
+		if (event.GetError())
+			LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+		break;
+	case CSocketEvent::connection:
+		if (event.GetError())
+		{
+			LogMessage(Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+			OnClose(event.GetError());
+		}
+		else
+		{
+			if (m_pProxyBackend && !m_pProxyBackend->Detached())
+			{
+				m_pProxyBackend->Detach();
+				m_pBackend = new CSocketBackend(this, m_pSocket);
+			}
 			OnConnect();
 		}
+		break;
+	case CSocketEvent::read:
 		OnReceive();
 		break;
-	case wxSOCKET_OUTPUT:
+	case CSocketEvent::write:
 		OnSend();
 		break;
-	case wxSOCKET_LOST:
-		OnClose();
+	case CSocketEvent::close:
+		OnClose(event.GetError());
+		break;
+	default:
+		LogMessage(Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
@@ -887,23 +1088,27 @@ void CRealControlSocket::OnSend()
 			return;
 		}
 
-		m_pBackend->Write(m_pSendBuffer, m_nSendBufferLen);
-		if (m_pBackend->Error())
+		int error;
+		int written = m_pBackend->Write(m_pSendBuffer, m_nSendBufferLen, error);
+		if (written < 0)
 		{
-			if (m_pBackend->LastError() != wxSOCKET_WOULDBLOCK)
+			if (error != EAGAIN)
+			{
+				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
+				if (GetCurrentCommandId() != cmd_connect)
+					LogMessage(::Error, _("Disconnected from server"));
 				DoClose();
+			}
 			return;
 		}
 
-		int numsent = m_pBackend->LastCount();
-
-		if (numsent)
+		if (written)
 		{
 			SetAlive();
-			m_pEngine->SetActive(false);
+			m_pEngine->SetActive(CFileZillaEngine::send);
 		}
 
-		if (numsent == m_nSendBufferLen)
+		if (written == m_nSendBufferLen)
 		{
 			m_nSendBufferLen = 0;
 			delete [] m_pSendBuffer;
@@ -911,18 +1116,23 @@ void CRealControlSocket::OnSend()
 		}
 		else
 		{
-			memmove(m_pSendBuffer, m_pSendBuffer + numsent, m_nSendBufferLen - numsent);
-			m_nSendBufferLen -= numsent;
+			memmove(m_pSendBuffer, m_pSendBuffer + written, m_nSendBufferLen - written);
+			m_nSendBufferLen -= written;
 		}
 	}
 }
 
-void CRealControlSocket::OnClose()
+void CRealControlSocket::OnClose(int error)
 {
-	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose()"));
+	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
 
 	if (GetCurrentCommandId() != cmd_connect)
-		LogMessage(::Error, _("Disconnected from server"));
+	{
+		if (!error)
+			LogMessage(::Error, _("Connection closed by server"));
+		else
+			LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+	}
 	DoClose();
 }
 
@@ -933,68 +1143,69 @@ int CRealControlSocket::Connect(const CServer &server)
 	if (server.GetEncodingType() == ENCODING_CUSTOM)
 		m_pCSConv = new wxCSConv(server.GetCustomEncoding());
 
-	if (m_pCurrentServer)
-		delete m_pCurrentServer;
+	delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
-	CConnectOpData* pData;
-	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
-		pData = 0;
-	else
-		pData = static_cast<CConnectOpData *>(m_pCurOpData);
-	
-	const wxString& host = pData ? pData->host : server.GetHost();
-	if (!IsIpAddress(host))
-	{
-		LogMessage(Status, _("Resolving IP-Address for %s"), host.c_str());
-		CAsyncHostResolver *resolver = new CAsyncHostResolver(m_pEngine, ConvertDomainName(host));
-		m_pEngine->AddNewAsyncHostResolver(resolver);
+	// International domain names
+	m_pCurrentServer->SetHost(ConvertDomainName(server.GetHost()), server.GetPort());
 
-		resolver->Create();
-		resolver->Run();
-	}
-	else
-	{
-		wxIPV4address addr;
-		addr.Hostname(host);
-		return ContinueConnect(&addr);
-	}
-
-	return FZ_REPLY_WOULDBLOCK;
+	return ContinueConnect();
 }
 
-int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
+int CRealControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CRealControlSocket::ContinueConnect(%p) m_pEngine=%p"), address, m_pEngine);
-	if (GetCurrentCommandId() != cmd_connect ||
-		!m_pCurrentServer)
-	{
-		LogMessage(Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
-		return DoClose(FZ_REPLY_INTERNALERROR);
-	}
-	
-	if (!address)
-	{
-		LogMessage(::Error, _("Invalid hostname or host not found"));
-		return DoClose(FZ_REPLY_ERROR | FZ_REPLY_CRITICALERROR);
-	}
+	wxString host;
+	unsigned int port = 0;
 
-	CConnectOpData* pData;
-	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
-		pData = 0;
+	const int proxy_type = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE);
+	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy())
+	{
+		LogMessage(::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost().c_str());
+
+		host = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_HOST);
+		port = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_PORT);
+
+		delete m_pBackend;
+		m_pProxyBackend = new CProxySocket(this, m_pSocket, this);
+		m_pBackend = m_pProxyBackend;
+		int res = m_pProxyBackend->Handshake((enum CProxySocket::ProxyType)proxy_type,
+											 m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
+											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_USER),
+											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_PASS));
+
+		if (res != EINPROGRESS)
+		{
+			LogMessage(::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res).c_str());
+			DoClose();
+			return FZ_REPLY_ERROR;
+		}
+	}
 	else
-		pData = static_cast<CConnectOpData *>(m_pCurOpData);
+	{
+		if (m_pCurOpData && m_pCurOpData->opId == cmd_connect)
+		{
+			CConnectOpData* pData(static_cast<CConnectOpData*>(m_pCurOpData));
+			host = ConvertDomainName(pData->host);
+			port = pData->port;
+		}
+		if (host == _T(""))
+		{
+			host = m_pCurrentServer->GetHost();
+			port = m_pCurrentServer->GetPort();
+		}
+	}
+	if (!IsIpAddress(host))
+		LogMessage(Status, _("Resolving address of %s"), host.c_str());
 
-	const unsigned int port = pData ? pData->port : m_pCurrentServer->GetPort();
-	LogMessage(Status, _("Connecting to %s:%d..."), address->IPAddress().c_str(), port);
+	int res = m_pSocket->Connect(host, port);
 
-	wxIPV4address addr = *address;
-	addr.Service(port);
-
-	bool res = wxSocketClient::Connect(addr, false);
-
-	if (!res && LastError() != wxSOCKET_WOULDBLOCK)
-		return DoClose();
+	// Treat success same as EINPROGRESS, we wait for connect notification in any case
+	if (res && res != EINPROGRESS)
+	{
+		LogMessage(::Error, _("Could not connect to server: %s"), CSocket::GetErrorDescription(res).c_str());
+		DoClose();
+		return FZ_REPLY_ERROR;
+	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -1002,13 +1213,13 @@ int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
 int CRealControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
 	ResetSocket();
-	
+
 	return CControlSocket::DoClose(nErrorCode);
 }
 
 void CRealControlSocket::ResetSocket()
 {
-	Close();
+	m_pSocket->Close();
 
 	if (m_pSendBuffer)
 	{
@@ -1017,26 +1228,213 @@ void CRealControlSocket::ResetSocket()
 		m_nSendBufferLen = 0;
 	}
 
-	m_onConnectCalled = false;
-
+	if (m_pProxyBackend)
+	{
+		if (m_pProxyBackend != m_pBackend)
+			delete m_pProxyBackend;
+		m_pProxyBackend = 0;
+	}
 	delete m_pBackend;
 	m_pBackend = 0;
 }
 
-wxString CRealControlSocket::GetLocalIP() const
+bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNotification)
 {
-	wxIPV4address addr;
-	if (!GetLocal(addr))
-		return _T("");
+	wxASSERT(pFileExistsNotification);
 
-	return addr.IPAddress();
+	if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
+		return false;
+	}
+
+	CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
+
+	switch (pFileExistsNotification->overwriteAction)
+	{
+	case CFileExistsNotification::overwrite:
+		SendNextCommand();
+		break;
+	case CFileExistsNotification::overwriteNewer:
+		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+			SendNextCommand();
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::overwriteSize:
+		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
+		Second compare flags the remaining case in which we need to send command : both size unknown */
+		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::overwriteSizeOrNewer:
+		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+			SendNextCommand();
+		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
+		Second compare flags the remaining case in which we need to send command : both size unknown */
+		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+			SendNextCommand();
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::resume:
+		if (pData->download && pData->localFileSize != -1)
+			pData->resume = true;
+		else if (!pData->download && pData->remoteFileSize != -1)
+			pData->resume = true;
+		SendNextCommand();
+		break;
+	case CFileExistsNotification::rename:
+		if (pData->download)
+		{
+			wxFileName fn = pData->localFile;
+			fn.SetFullName(pFileExistsNotification->newName);
+			pData->localFile = fn.GetFullPath();
+
+			wxLongLong size;
+			bool isLink;
+			if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
+				pData->localFileSize = size.GetValue();
+			else
+				pData->localFileSize = -1;
+
+			if (CheckOverwriteFile() == FZ_REPLY_OK)
+				SendNextCommand();
+		}
+		else
+		{
+			pData->remoteFile = pFileExistsNotification->newName;
+
+			CDirectoryCache cache;
+
+			CDirentry entry;
+			bool dir_did_exist;
+			bool matched_case;
+			if (cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
+				matched_case)
+			{
+				wxLongLong size = entry.size;
+				pData->remoteFileSize = size.GetLo() + ((wxFileOffset)size.GetHi() << 32);
+				if (entry.has_date())
+					pData->fileTime = entry.time;
+
+				if (CheckOverwriteFile() != FZ_REPLY_OK)
+					break;
+			}
+			else
+			{
+				pData->fileTime = wxDateTime();
+				pData->remoteFileSize = -1;
+			}
+
+			SendNextCommand();
+		}
+		break;
+	case CFileExistsNotification::skip:
+		if (pData->download)
+		{
+			wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+			LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+		}
+		else
+		{
+			LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+		}
+		ResetOperation(FZ_REPLY_OK);
+		break;
+	default:
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return false;
+	}
+
+	return true;
 }
 
-wxString CRealControlSocket::GetPeerIP() const
+void CControlSocket::CreateLocalDir(const wxString &local_file)
 {
-	wxIPV4address addr;
-	if (!GetPeer(addr))
-		return _T("");
+	wxString file;
+	CLocalPath local_path(local_file, &file);
+	if (local_path.empty() || !local_path.HasParent())
+		return;
 
-	return addr.IPAddress();
+	// Only go back as far as needed. On comparison, wxWidgets'
+	// wxFileName::Mkdir always starts at the root.
+	std::list<wxString> segments;
+	while (!local_path.Exists() && local_path.HasParent())
+	{
+		wxString segment;
+		local_path.MakeParent(&segment);
+		segments.push_front(segment);
+	}
+
+	CLocalPath last_successful;
+	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
+	{
+		local_path.AddSegment(*iter);
+
+#ifdef __WXMSW__
+		BOOL res = CreateDirectory(local_path.GetPath(), 0);
+		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
+			break;
+#else
+		const wxCharBuffer s = local_path.GetPath().fn_str();
+
+		int res = mkdir(s, 0777);
+		if (res && errno != EEXIST)
+			break;
+#endif
+		last_successful = local_path;
+	}
+
+	if (last_successful.empty())
+		return;
+
+	// Send out notification
+	CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
+	n->dir = last_successful;
+	m_pEngine->AddNotification(n);
 }
