@@ -380,7 +380,16 @@ bool CLocalPath::ChangePath(const wxString& path)
 	if (m_path.empty())
 		return false;
 
-	return SetPath(m_path + path);
+	if ((path[0] == '\\' || path[0] == '/') && m_path[1] == ':')
+	{
+		// Relative to drive root
+		return SetPath(m_path.Left(2) + path);
+	}
+	else
+	{
+		// Relative to current directory
+		return SetPath(m_path + path);
+	}
 #else
 	if (path[0] == path_separator)
 	{
@@ -414,9 +423,7 @@ bool CLocalPath::Exists(wxString *error /*=0*/) const
 
 	if (m_path[0] == '\\')
 	{
-		// UNC path
-
-		// \\server\share\ 
+		// \\server\share\ UNC path
 
 		size_t pos;
 
