@@ -1,18 +1,18 @@
 #include <filezilla.h>
-
-#include "LocalTreeView.h"
-#include "LocalListView.h"
-#include "RemoteTreeView.h"
-#include "RemoteListView.h"
-#include "StatusView.h"
-#include "queue.h"
 #include "Mainfrm.h"
+
+#include "LocalListView.h"
+#include "LocalTreeView.h"
+#include "queue.h"
+#include "RemoteListView.h"
+#include "RemoteTreeView.h"
+#include "StatusView.h"
 #include "state.h"
 #include "Options.h"
-#include "commandqueue.h"
 #include "asyncrequestqueue.h"
+#include "commandqueue.h"
 #include "led.h"
-#include "sitemanager.h"
+#include "sitemanager_dialog.h"
 #include "settings/settingsdialog.h"
 #include "themeprovider.h"
 #include "filezillaapp.h"
@@ -519,8 +519,7 @@ bool CMainFrame::CreateMenus()
 
 bool CMainFrame::CreateQuickconnectBar()
 {
-	if (m_pQuickconnectBar)
-		delete m_pQuickconnectBar;
+	delete m_pQuickconnectBar;
 
 	m_pQuickconnectBar = new CQuickconnectBar();
 	if (!m_pQuickconnectBar->Create(this))
@@ -1341,14 +1340,14 @@ void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 	if (!pState)
 		return;
 
-	CSiteManager dlg;
+	CSiteManagerDialog dlg;
 
 	std::set<wxString> handled_paths;
-	std::vector<CSiteManager::_connected_site> connected_sites;
+	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
 	if (pServer)
 	{
-		CSiteManager::_connected_site connected_site;
+		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.server = *pServer;
 		connected_sites.push_back(connected_site);
 	}
@@ -1365,7 +1364,7 @@ void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 		if (handled_paths.find(path) != handled_paths.end())
 			continue;
 
-		CSiteManager::_connected_site connected_site;
+		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
 		connected_site.server = controls->pState->GetLastServer();
 		connected_sites.push_back(connected_site);
@@ -1393,7 +1392,7 @@ void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
 				controls->site_bookmarks->path = connected_sites[j].new_path;
 
 				controls->site_bookmarks->bookmarks.clear();
-				dlg.GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
+				CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
 
 				break;
 			}
@@ -1766,15 +1765,9 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 	if (swap == -1)
 		swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
 
-	bool final;
 	if (messagelog_position == -1)
-	{
-		final = true;
 		messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
-	}
-	else
-		final = false;
-
+	
 	// First handle changes in message log position as it can make size of the other panes change
 	{
 		bool shown = m_pStatusView->IsShown();
@@ -2017,13 +2010,6 @@ void CMainFrame::CheckChangedSettings()
 
 	m_pAsyncRequestQueue->RecheckDefaults();
 
-	CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
-	if (controls)
-	{
-		controls->pLocalListView->InitDateFormat();
-		controls->pRemoteListView->InitDateFormat();
-	}
-
 	m_pQueueView->SettingsChanged();
 	CAutoAsciiFiles::SettingsChanged();
 }
