@@ -34,7 +34,8 @@ enum Flags
 {
 	normal,
 	internal,
-	default_only
+	default_only,
+	default_priority // If that option is given in fzdefaults.xml, it overrides any user option
 };
 
 struct t_Option
@@ -94,12 +95,12 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Trusted root certificate", string, _T(""), internal },
 	{ "Size format", number, _T("0"), normal },
 	{ "Size thousands separator", number, _T("1"), normal },
-	{ "Size decimal places", number, _T("0"), normal },
+	{ "Size decimal places", number, _T("1"), normal },
 
 	// Interface settings
 	{ "Number of Transfers", number, _T("2"), normal },
 	{ "Ascii Binary mode", number, _T("0"), normal },
-	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|js|jsp|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
+	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
 	{ "Auto Ascii no extension", number, _T("1"), normal },
 	{ "Auto Ascii dotfiles", number, _T("1"), normal },
 	{ "Theme", string, _T("opencrystal/"), normal },
@@ -177,7 +178,7 @@ static const t_Option options[OPTIONS_NUM] =
 
 	// Default/internal options
 	{ "Config Location", string, _T(""), default_only },
-	{ "Kiosk mode", number, _T("0"), default_only },
+	{ "Kiosk mode", number, _T("0"), default_priority },
 	{ "Disable update check", number, _T("0"), default_only }
 };
 
@@ -325,6 +326,14 @@ bool COptions::SetOption(unsigned int nID, wxString value)
 	return true;
 }
 
+bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
+{
+	if (nID >= OPTIONS_NUM)
+		return false;
+
+	return m_optionsCache[nID].from_default;
+}
+
 void COptions::CreateSettingsXmlElement()
 {
 	if (!m_pXmlFile)
@@ -640,6 +649,8 @@ void COptions::Import(TiXmlElement* pElement)
 	std::map<std::string, int> nameOptionMap;
 	GetNameOptionMap(nameOptionMap);
 	LoadOptions(nameOptionMap, pElement);
+	if (!m_save_timer.IsRunning())
+		m_save_timer.Start(15000, true);
 }
 
 void COptions::LoadOptions(const std::map<std::string, int>& nameOptionMap, TiXmlElement* settings /*=0*/)
@@ -682,7 +693,6 @@ void COptions::LoadOptionFromElement(TiXmlElement* pOption, const std::map<std::
 	{
 		if (!allowDefault && options[iter->second].flags == default_only)
 			return;
-
 		wxString value;
 
 		TiXmlNode *text = pOption->FirstChild();
@@ -693,6 +703,17 @@ void COptions::LoadOptionFromElement(TiXmlElement* pOption, const std::map<std::
 			value = ConvLocal(text->Value());
 		}
 
+		if (options[iter->second].flags == default_priority)
+		{
+			if (allowDefault)
+				m_optionsCache[iter->second].from_default = true;
+			else
+			{
+				if (m_optionsCache[iter->second].from_default)
+					return;
+			}
+		}
+
 		if (options[iter->second].type == number)
 		{
 			long numValue = 0;
@@ -829,5 +850,6 @@ void COptions::SetDefaultValues()
 			options[i].defaultValue.ToLong(&numValue);
 			m_optionsCache[i].numValue = numValue;
 		}
+		m_optionsCache[i].from_default = false;
 	}
 }
