@@ -81,23 +81,18 @@ enum interfaceOptions
 	OPTION_SPEED_DISPLAY,
 	OPTION_TOOLBAR_HIDDEN,
 
+	// Default/internal options
+	OPTION_DEFAULT_SETTINGSDIR,
+	OPTION_DEFAULT_KIOSKMODE,
+	OPTION_DEFAULT_DISABLEUPDATECHECK,
+
 	// Has to be last element
 	OPTIONS_NUM
 };
 
-enum defaultOptions
-{
-	DEFAULT_SETTINGSDIR,
-	DEFAULT_KIOSKMODE,
-	DEFAULT_DISABLEUPDATECHECK,
-
-	// Has to be last element
-	DEFAULTS_NUM
-};
-
 struct t_OptionsCache
 {
-	bool cached;
+	bool from_default;
 	int numValue;
 	wxString strValue;
 };
@@ -110,11 +105,10 @@ public:
 	virtual int GetOptionVal(unsigned int nID);
 	virtual wxString GetOption(unsigned int nID);
 
-	int GetDefaultVal(unsigned int nID) const;
-	wxString GetDefault(unsigned int nID) const;
-
 	virtual bool SetOption(unsigned int nID, int value);
 	virtual bool SetOption(unsigned int nID, wxString value);
+
+	bool OptionFromFzDefaultsXml(unsigned int nID);
 	
 	void SetLastServer(const CServer& server);
 	bool GetLastServer(CServer& server);
@@ -135,7 +129,6 @@ protected:
 	wxString Validate(unsigned int nID, wxString value);
 
 	void SetXmlValue(unsigned int nID, wxString value);
-	bool GetXmlValue(unsigned int nID, wxString &value, TiXmlElement* settings = 0);
 
 	// path is element path below document root, separated by slashes
 	void SetServer(wxString path, const CServer& server);
@@ -143,7 +136,12 @@ protected:
 
 	void CreateSettingsXmlElement();
 
-	void LoadGlobalDefaultOptions();
+	void GetNameOptionMap(std::map<std::string, int>& nameOptionMap) const; //Give me C++1x
+	void LoadOptions(const std::map<std::string, int>& nameOptionMap, TiXmlElement* settings = 0);
+	void LoadGlobalDefaultOptions(const std::map<std::string, int>& nameOptionMap);
+	void LoadOptionFromElement(TiXmlElement* pOption, const std::map<std::string, int>& nameOptionMap, bool allowDefault);
+	void InitSettingsDir();
+	void SetDefaultValues();
 
 	void Save();
 
