@@ -471,7 +471,9 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	m_actionAfterTimerId = -1;
 #endif
 
-	CreateColumns(_("Status"));
+	std::list<ColumnId> extraCols;
+	extraCols.push_back(colTransferStatus);
+	CreateColumns(extraCols);
 
 	//Initialize the engine data
 	t_EngineData *pData = new t_EngineData;
@@ -835,7 +837,6 @@ bool CQueueView::TryStartNextTransfer()
 		CFileItem* fileItem;
 		CServerItem* serverItem;
 		t_EngineData* pEngineData;
-		bool browsingConnection;
 	} bestMatch = {0};
 
 	// Find inactive file. Check all servers for
@@ -1920,7 +1921,7 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 
 void CQueueView::SaveQueue()
 {
-	// Kiosk mode doesn't save queue
+	// Kiosk mode 2 doesn't save queue
 	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 		return;
 
@@ -2009,7 +2010,11 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 				wxLongLong size = GetTextElementLongLong(pFile, "Size", -1);
 				unsigned int errorCount = GetTextElementInt(pFile, "ErrorCount");
 				unsigned int priority = GetTextElementInt(pFile, "Priority", priority_normal);
-				bool binary = GetTextElementInt(pFile, "TransferMode", 1) != 0;
+
+				int dataType = GetTextElementInt(pFile, "DataType", -1);
+				if (dataType == -1)
+					dataType = GetTextElementInt(pFile, "TransferMode", 1); 
+				bool binary = dataType != 0;
 				int overwrite_action = GetTextElementInt(pFile, "OverwriteAction", CFileExistsNotification::unknown);
 
 				CServerPath remotePath;
@@ -2137,12 +2142,7 @@ void CQueueView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-#ifndef __WXMSW__
-	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	const bool has_selection = GetSelectedItemCount() != 0;
-#else
-	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
-#endif
+	bool has_selection = HasSelection();
 
 	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
 	pMenu->Check(XRCID("ID_ACTIONAFTER_DISABLE"), IsActionAfter(ActionAfterState_Disabled));
@@ -2460,13 +2460,7 @@ void CQueueView::SetDefaultFileExistsAction(enum CFileExistsNotification::Overwr
 
 void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &event)
 {
-#ifndef __WXMSW__
-	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	const bool has_selection = GetSelectedItemCount() != 0;
-#else
-	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
-#endif
-	if (!has_selection)
+	if (!HasSelection())
 		return;
 
 	CDefaultFileExistsDlg dlg;
