@@ -18,6 +18,7 @@
 #include "dragdropmanager.h"
 #include <wx/clipbrd.h>
 #include "sizeformatting.h"
+#include "timeformatting.h"
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
@@ -367,18 +368,15 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	m_dirIcon = GetIconIndex(dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-	InitHeaderImageList();
+	InitHeaderSortImageList();
 
 	SetDirectoryListing(0);
 
 	SetDropTarget(new CRemoteListViewDropTarget(this));
 
-	InitDateFormat();
-
 	EnablePrefixSearch(true);
 
 	m_pLinkResolveState = 0;
-
 }
 
 CRemoteListView::~CRemoteListView()
@@ -2438,7 +2436,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 	}
 #endif
 
-	CLabelEditBlocker(*this);
+	CLabelEditBlocker b(*this);
 
 	wxDropSource source(this);
 	source.SetData(object);
@@ -2528,25 +2526,6 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 #endif
 }
 
-void CRemoteListView::InitDateFormat()
-{
-	const wxString& dateFormat = COptions::Get()->GetOption(OPTION_DATE_FORMAT);
-	const wxString& timeFormat = COptions::Get()->GetOption(OPTION_TIME_FORMAT);
-
-	if (dateFormat == _T("1"))
-		m_dateFormat = _T("%Y-%m-%d");
-	else if (dateFormat[0] == '2')
-		m_dateFormat = dateFormat.Mid(1);
-	else
-		m_dateFormat = _T("%x");
-
-	if (timeFormat == _T("1"))
-		m_timeFormat = m_dateFormat + _T(" %H:%M");
-	else if (timeFormat[0] == '2')
-		m_timeFormat = m_dateFormat + _T(" ") + timeFormat.Mid(1);
-	else
-		m_timeFormat = m_dateFormat + _T(" %X");
-}
 
 void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 {
@@ -2661,9 +2640,9 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 			wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
 			continue;
 		case CEditHandler::removing:
-			if (!pEditHandler)
+			if (!pEditHandler->Remove(entry.name, path, server))
 			{
-				wxMessageBox(_("A file with that name is still being edited. Please close it and try again."), _("Selected file already opened."), wxICON_EXCLAMATION);
+				wxMessageBox(_("A file with that name is still being edited. Please close it and try again."), _("Selected file is already opened"), wxICON_EXCLAMATION);
 				continue;
 			}
 			break;
@@ -2691,7 +2670,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 				{
 					if (!pEditHandler->Remove(entry.name, path, server))
 					{
-						wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file still being edited"), wxICON_EXCLAMATION);
+						wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file is still being edited"), wxICON_EXCLAMATION);
 						continue;
 					}
 				}
@@ -2721,7 +2700,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 void CRemoteListView::ValidateIndexMapping()
 {
 	// This ensures that the index mapping is a bijection.
-    // Beware:
+	// Beware:
 	// - NO filter may be used!
 	// - Doesn't work in comparison mode
 
@@ -2947,9 +2926,9 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 			return _T("");
 
 		if (entry.has_time())
-			return entry.time.Format(m_timeFormat);
+			return CTimeFormat::FormatDateTime(entry.time);
 		else
-			return entry.time.Format(m_dateFormat);
+			return CTimeFormat::FormatDate(entry.time);
 	}
 	else if (column == 4)
 		return (*m_pDirectoryListing)[index].permissions;
