@@ -4,6 +4,8 @@
 #include "queueview_failed.h"
 #include "queueview_successful.h"
 #include "sizeformatting.h"
+#include "timeformatting.h"
+#include "themeprovider.h"
 
 CQueueItem::CQueueItem()
 {
@@ -356,7 +358,7 @@ void CFileItem::SaveItem(TiXmlElement* pElement) const
 		AddTextElement(file, "ErrorCount", m_errorCount);
 	if (m_priority != priority_normal)
 		AddTextElement(file, "Priority", m_priority);
-	AddTextElementRaw(file, "TransferMode", m_transferSettings.binary ? "1" : "0");
+	AddTextElementRaw(file, "DataType", m_transferSettings.binary ? "1" : "0");
 	if (m_defaultFileExistsAction != CFileExistsNotification::unknown)
 		AddTextElement(file, "OverwriteAction", m_defaultFileExistsAction);
 }
@@ -779,12 +781,13 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 	m_folderScanCountChanged = false;
 
 	// Create and assign the image list for the queue
-	wxImageList* pImageList = new wxImageList(16, 16);
+	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
+	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FILE"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"),  wxART_OTHER, wxSize(16, 16)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
 
 	AssignImageList(pImageList, wxIMAGE_LIST_SMALL);
 
@@ -841,12 +844,20 @@ void CQueueViewBase::OnEraseBackground(wxEraseEvent& event)
 
 wxString CQueueViewBase::OnGetItemText(long item, long column) const
 {
+	if (column < 0 || static_cast<std::size_t>(column) >= m_columns.size())
+		return _T("");
+
 	CQueueViewBase* pThis = const_cast<CQueueViewBase*>(this);
 
 	CQueueItem* pItem = pThis->GetQueueItem(item);
 	if (!pItem)
 		return _T("");
 
+	return OnGetItemText(pItem, m_columns[column]);
+}
+
+wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
+{
 	switch (pItem->GetType())
 	{
 	case QueueItemType_Server:
@@ -861,9 +872,9 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
 			switch (column)
 			{
-			case 0:
+			case colLocalName:
 				return pFileItem->GetIndent() + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
-			case 1:
+			case colDirection:
 				if (pFileItem->Download())
 					if (pFileItem->queued())
 						return _T("<--");
@@ -875,9 +886,9 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					else
 						return _T("-->>");
 				break;
-			case 2:
+			case colRemoteName:
 				return pFileItem->GetRemotePath().FormatFilename(pFileItem->GetRemoteFile());
-			case 3:
+			case colSize:
 				{
 					const wxLongLong& size = pFileItem->GetSize();
 					if (size >= 0)
@@ -885,7 +896,7 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					else
 						return _T("?");
 				}
-			case 4:
+			case colPriority:
 				switch (pFileItem->GetPriority())
 				{
 				case 0:
@@ -901,8 +912,11 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					return _("Highest");
 				}
 				break;
-			case 5:
+			case colTransferStatus:
+			case colErrorReason:
 				return pFileItem->m_statusMessage;
+			case colTime:
+				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
 				break;
 			}
@@ -913,9 +927,9 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			CFolderScanItem* pFolderItem = reinterpret_cast<CFolderScanItem*>(pItem);
 			switch (column)
 			{
-			case 0:
+			case colLocalName:
 				return _T("  ") + pFolderItem->GetLocalPath().GetPath();
-			case 1:
+			case colDirection:
 				if (pFolderItem->Download())
 					if (pFolderItem->queued())
 						return _T("<--");
@@ -927,10 +941,13 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					else
 						return _T("-->>");
 				break;
-			case 2:
+			case colRemoteName:
 				return pFolderItem->GetRemotePath().GetPath();
-			case 5:
+			case colTransferStatus:
+			case colErrorReason:
 				return pFolderItem->m_statusMessage;
+			case colTime:
+				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
 				break;
 			}
@@ -941,11 +958,11 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			CFileItem* pFolderItem = reinterpret_cast<CFolderItem*>(pItem);
 			switch (column)
 			{
-			case 0:
+			case colLocalName:
 				if (pFolderItem->Download())
 					return pFolderItem->GetIndent() + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
 				break;
-			case 1:
+			case colDirection:
 				if (pFolderItem->Download())
 					if (pFolderItem->queued())
 						return _T("<--");
@@ -957,7 +974,7 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					else
 						return _T("-->>");
 				break;
-			case 2:
+			case colRemoteName:
 				if (!pFolderItem->Download())
 				{
 					if (pFolderItem->GetRemoteFile() == _T(""))
@@ -966,7 +983,7 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 						return pFolderItem->GetRemotePath().FormatFilename(pFolderItem->GetRemoteFile());
 				}
 				break;
-			case 4:
+			case colPriority:
 				switch (pFolderItem->GetPriority())
 				{
 				case 0:
@@ -982,8 +999,11 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 					return _("Highest");
 				}
 				break;
-			case 5:
+			case colTransferStatus:
+			case colErrorReason:
 				return pFolderItem->m_statusMessage;
+			case colTime:
+				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
 				break;
 			}
@@ -1184,17 +1204,26 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 		SetItemState(*iter, 0, wxLIST_STATE_SELECTED);
 }
 
-void CQueueViewBase::CreateColumns(const wxString& lastColumnName)
+void CQueueViewBase::AddQueueColumn(ColumnId id)
 {
-	const unsigned long widths[6] = { 180, 60, 180, 80, 60, 150 };
+	const unsigned long widths[8] = { 180, 60, 180, 80, 60, 100, 150, 150 };
+	const int alignment[8] = { wxLIST_FORMAT_LEFT, wxLIST_FORMAT_CENTER, wxLIST_FORMAT_LEFT, wxLIST_FORMAT_RIGHT, wxLIST_FORMAT_LEFT, wxLIST_FORMAT_LEFT, wxLIST_FORMAT_LEFT, wxLIST_FORMAT_LEFT };
+	const wxString names[8] = { _("Server/Local file"), _("Direction"), _("Remote file"), _("Size"), _("Priority"), _("Time"), _("Status"), _("Reason") };
 
-	AddColumn(_("Server/Local file"), wxLIST_FORMAT_LEFT, widths[0]);
-	AddColumn(_("Direction"), wxLIST_FORMAT_CENTER, widths[1]);
-	AddColumn(_("Remote file"), wxLIST_FORMAT_LEFT, widths[2]);
-	AddColumn(_("Size"), wxLIST_FORMAT_RIGHT, widths[3]);
-	AddColumn(_("Priority"), wxLIST_FORMAT_LEFT, widths[4]);
-	if (lastColumnName != _T(""))
-		AddColumn(lastColumnName, wxLIST_FORMAT_LEFT, widths[5]);
+	AddColumn(names[id], alignment[id], widths[id]);
+	m_columns.push_back(id);
+}
+
+void CQueueViewBase::CreateColumns(std::list<ColumnId> const& extraColumns)
+{
+	AddQueueColumn(colLocalName);
+	AddQueueColumn(colDirection);
+	AddQueueColumn(colRemoteName);
+	AddQueueColumn(colSize);
+	AddQueueColumn(colPriority);
+
+	for( std::list<ColumnId>::const_iterator it = extraColumns.begin(); it != extraColumns.end(); ++it)
+		AddQueueColumn(*it);
 
 	LoadColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
 }
@@ -1284,6 +1313,8 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 {
 	const int newIndex = GetItemIndex(pServerItem) + pServerItem->GetChildrenCount(true) + 1;
 
+	pItem->UpdateTime();
+
 	pServerItem->AddChild(pItem);
 	m_itemCount++;
 
